package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public class NoekeonEngine implements BlockCipher {
   private static final int genericSize = 16;
   private static final int[] nullVector = new int[4];
   private static final int[] roundConstants = new int[]{128, 27, 54, 108, 216, 171, 77, 154, 47, 94, 188, 99, 198, 151, 53, 106, 212};
   private int[] state = new int[4];
   private int[] subKeys = new int[4];
   private int[] decryptKeys = new int[4];
   private boolean _initialised = false;
   private boolean _forEncryption;

   private void setKey(byte[] key) {
      this.subKeys[0] = this.bytesToIntBig(key, 0);
      this.subKeys[1] = this.bytesToIntBig(key, 4);
      this.subKeys[2] = this.bytesToIntBig(key, 8);
      this.subKeys[3] = this.bytesToIntBig(key, 12);
   }

   private int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      this.state[0] = this.bytesToIntBig(in, inOff);
      this.state[1] = this.bytesToIntBig(in, inOff + 4);
      this.state[2] = this.bytesToIntBig(in, inOff + 8);
      this.state[3] = this.bytesToIntBig(in, inOff + 12);

      int in1;
      for(in1 = 0; in1 < 16; ++in1) {
         int[] inOff1;
         int[] var10008 = inOff1 = this.state;
         var10008[0] ^= roundConstants[in1];
         this.theta(inOff1, this.subKeys);
         this.pi1(this.state);
         this.gamma(this.state);
         this.pi2(this.state);
      }

      int[] var5;
      int[] var10010 = var5 = this.state;
      var10010[0] ^= roundConstants[in1];
      this.theta(var5, this.subKeys);
      this.intToBytesBig(this.state[0], out, outOff);
      this.intToBytesBig(this.state[1], out, outOff + 4);
      this.intToBytesBig(this.state[2], out, outOff + 8);
      this.intToBytesBig(this.state[3], out, outOff + 12);
      return 16;
   }

   private int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      this.state[0] = this.bytesToIntBig(in, inOff);
      this.state[1] = this.bytesToIntBig(in, inOff + 4);
      this.state[2] = this.bytesToIntBig(in, inOff + 8);
      this.state[3] = this.bytesToIntBig(in, inOff + 12);
      int[] var10002 = this.subKeys;
      int[] in1 = this.decryptKeys;
      inOff = var10002.length;
      System.arraycopy(var10002, 0, in1, 0, inOff);
      this.theta(this.decryptKeys, nullVector);

      int in2;
      for(in2 = 16; in2 > 0; --in2) {
         int[] inOff1 = this.state;
         this.theta(inOff1, this.decryptKeys);
         int[] var10005 = this.state;
         var10005[0] ^= roundConstants[in2];
         this.pi1(var10005);
         this.gamma(this.state);
         this.pi2(this.state);
      }

      int[] var6 = this.state;
      this.theta(var6, this.decryptKeys);
      int[] var9 = this.state;
      var9[0] ^= roundConstants[in2];
      this.intToBytesBig(var9[0], out, outOff);
      this.intToBytesBig(this.state[1], out, outOff + 4);
      this.intToBytesBig(this.state[2], out, outOff + 8);
      this.intToBytesBig(this.state[3], out, outOff + 12);
      return 16;
   }

   private void gamma(int[] a) {
      a[1] ^= ~a[3] & ~a[2];
      a[0] ^= a[2] & a[1];
      int var2 = a[3];
      a[3] = a[0];
      a[0] = var2;
      a[2] ^= a[0] ^ a[1] ^ a[3];
      a[1] ^= ~a[3] & ~a[2];
      a[0] ^= a[2] & a[1];
   }

   private void theta(int[] a, int[] k) {
      int var3 = (var3 = a[0] ^ a[2]) ^ this.rotl(var3, 8) ^ this.rotl(var3, 24);
      a[1] ^= var3;
      a[3] ^= var3;

      for(var3 = 0; var3 < 4; ++var3) {
         a[var3] ^= k[var3];
      }

      int a1;
      int var4 = (a1 = a[1] ^ a[3]) ^ this.rotl(a1, 8) ^ this.rotl(a1, 24);
      a[0] ^= var4;
      a[2] ^= var4;
   }

   private void pi1(int[] a) {
      a[1] = this.rotl(a[1], 1);
      a[2] = this.rotl(a[2], 5);
      a[3] = this.rotl(a[3], 2);
   }

   private void pi2(int[] a) {
      a[1] = this.rotl(a[1], 31);
      a[2] = this.rotl(a[2], 27);
      a[3] = this.rotl(a[3], 30);
   }

   private int bytesToIntBig(byte[] in, int off) {
      int var3 = off + 1;
      return in[off] << 24 | (in[var3++] & 255) << 16 | (in[var3++] & 255) << 8 | in[var3] & 255;
   }

   private void intToBytesBig(int x, byte[] out, int off) {
      int var10001 = x;
      int var10003 = x;
      int var10005 = x;
      int var4 = off + 1;
      out[off] = (byte)(x >>> 24);
      x = var4 + 1;
      out[var4] = (byte)(var10005 >>> 16);
      var4 = x + 1;
      out[x] = (byte)(var10003 >>> 8);
      out[var4] = (byte)var10001;
   }

   private int rotl(int x, int y) {
      return x << y | x >>> 32 - y;
   }

   public String getAlgorithmName() {
      return "Noekeon";
   }

   public int getBlockSize() {
      return 16;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof KeyParameter) {
         this._forEncryption = forEncryption;
         this._initialised = true;
         this.setKey(((KeyParameter)params).getKey());
      } else {
         throw new IllegalArgumentException("invalid parameter passed to Noekeon init - " + params.getClass().getName());
      }
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this._initialised) {
         if (inOff + 16 <= in.length) {
            if (outOff + 16 <= out.length) {
               return this._forEncryption ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
      }
   }

   public void reset() {
   }
}
